/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.reporter;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;



/**
 * Class with entry point to the ThermostatQA test reporting tool.
 *
 * @author Pavel Tisnovsky <ptisnovs@redhat.com>
 */
public class Reporter
{
    /**
     * Contents of log all files.
     */
    private Map<String, Set<String>> logFiles = new TreeMap<String, Set<String>>();

    /**
     * Test results read from log files.
     */
    private Map<String, Map<String, List<String>>> testResults = new TreeMap<String, Map<String, List<String>>>();

    /**
     * Set containing all tests names.
     */
    private Set<String> allTestNames = new TreeSet<String>();

    /**
     * All preprocessed command line parameters.
     */
    private CommandLineParameters params = null;

    /**
     * Constructor which preprocess all command line parameters.
     * 
     * @param args
     *            command line parameters
     */
    public Reporter(String[] args)
    {
        this.params = new CommandLineParameters(args);
    }

    /**
     * Run test reporting tool.
     */
    private void run()
    {
        readListOfAllLogFiles();
        readTestResults();
        createListOfAllTests();
        try
        {
            IndexPageGenerator.generate(this.testResults, this.params);
        }
        catch (RuntimeException e)
        {
            e.printStackTrace();
        }
        HistoryPagesGenerator.generate(this.testResults, this.params);
        GraphPagesGenerator.generate(this.testResults, this.params);
        LogPagesGenerator.generate(this.testResults, this.params);
        try
        {
            ResultsGraphGenerator.generate(this.testResults, this.params);
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Read list of all log files for all dates.
     */
    private void readListOfAllLogFiles()
    {
        File logDirectory = new File(this.params.getLogDir());
        System.out.println("logDir: " + logDirectory.getAbsolutePath());
        // process all subdirectories
        for (String subDir : logDirectory.list())
        {
            readListOfAllLogFilesForGivenDate(logDirectory, subDir);
        }
        System.out.println(this.logFiles);
    }

    /**
     * Read list of all log files for given date.
     * 
     * @param logDirectory
     *            directory where subdirectories with log files are stored
     * @param subDir
     *            subdirectory name
     */
    private void readListOfAllLogFilesForGivenDate(File logDirectory, String subDir)
    {
        File logSubDir = new File(logDirectory, subDir);
        Set<String> logFileSet = new TreeSet<String>();
        System.out.println("    subDir: " + logSubDir.getAbsolutePath());
        for (String logFileName : logSubDir.list())
        {
            readAbsolutePathForOneLogFile(logSubDir, logFileSet, logFileName);
        }
        // subDir == date
        this.logFiles.put(subDir, logFileSet);
    }

    /**
     * Read absolute path of one log file. This path is then stored into
     * {@link #logFiles}
     * 
     * @param logSubDir
     *            directory where log files are stored
     * @param logFileSet
     *            set into which absolute path is added
     * @param logFileName
     *            log file name
     */
    private void readAbsolutePathForOneLogFile(File logSubDir, Set<String> logFileSet, String logFileName)
    {
        File logFile = new File(logSubDir, logFileName);
        System.out.println("        logFile: " + logFile.getPath());
        logFileSet.add(logFile.getAbsolutePath());
    }

    /**
     * Read all test results from log files.
     */
    private void readTestResults()
    {
        // iterate over names of log files
        for (Map.Entry<String, Set<String>> logFileEntry : this.logFiles.entrySet())
        {
            String date = logFileEntry.getKey();
            Set<String> fileNames = logFileEntry.getValue();
            Map<String, List<String>> testResultsForOneDate = new TreeMap<String, List<String>>();
            // read content of each log file
            for (String fileName : fileNames)
            {
                List<String> logFileContent = FileUtils.readTextFile(fileName);
                testResultsForOneDate.put(fileName, logFileContent);
            }
            this.testResults.put(date, testResultsForOneDate);
        }
    }

    /**
     * Fill up the set allTestNames which will contain names of all tests.
     * Because this set is implemented as TreeSet, the test names are
     * alphabetically sorted.
     */
    private void createListOfAllTests()
    {
        // iterate over all test results
        for (Map.Entry<String, Map<String, List<String>>> testResult : this.testResults.entrySet())
        {
            // each result consists of a pair name:content_of_log_file
            for (Map.Entry<String, List<String>> testResultContent : testResult.getValue().entrySet())
            {
                // ATM we are interested only in real test results, not other lines
                for (String oneLogLine : testResultContent.getValue())
                {
                    if (oneLogLine.startsWith("PASSED: "))
                    {
                        String testName = oneLogLine.substring("PASSED: ".length());
                        this.allTestNames.add(testName);
                    }
                    if (oneLogLine.startsWith("FAILED: "))
                    {
                        String testName = oneLogLine.substring("FAILED: ".length());
                        testName = testName.substring(0, testName.indexOf(": FAILED"));
                        this.allTestNames.add(testName);
                    }
                    /*
                    if (oneLogLine.startsWith("ERROR: "))
                    {
                        String testName = oneLogLine.substring("ERROR: ".length());
                        this.allTestNames.add(testName);
                    }
                    if (oneLogLine.startsWith("IGNORED: "))
                    {
                        String testName = oneLogLine.substring("IGNORED: ".length());
                        this.allTestNames.add(testName);
                    }*/
                }
            }
        }
    }

    /**
     * Entry point to the ThermostatQA test reporting tool.
     * 
     * @param args
     *            command line parameters
     */
    public static void main(String[] args)
    {
        new Reporter(args).run();
    }

}
